/*Written with Emacs.*/
/*
  GtkDiskFree shows free space on your mounted partitions.
  Copyright (C) 2001-2002  Dj-Death (Landwerlin Lionel)
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "main.h"
#include "callbacks.h"
#include "diskfree.h"
#include "interface.h"
#include "menus.h"


gboolean
affichage_status_bar_block_size (void)
{
  gchar *message;

  gtk_statusbar_pop(GTK_STATUSBAR(status_bar_block), block_size_status);

  switch (options->blocks)
    {
    case 'h':
      message = g_strdup_printf(_(" Human readable"));
      gtk_statusbar_push(GTK_STATUSBAR(status_bar_block),
			 block_size_status, message);
      break;
    case 'g':
      message = g_strdup_printf(_(" Gigabytes"));
      gtk_statusbar_push(GTK_STATUSBAR(status_bar_block),
			 block_size_status, message);    
      break;
    case 'm':
      message = g_strdup_printf(_(" Megabytes"));
      gtk_statusbar_push(GTK_STATUSBAR(status_bar_block),
			 block_size_status, message);
      break;
    case 'k':
      message = g_strdup_printf(_(" Kilobytes"));
      gtk_statusbar_push(GTK_STATUSBAR(status_bar_block),
			 block_size_status, message);
      break;
    default:
      message = g_strdup_printf(_(" Human readable"));
      gtk_statusbar_push(GTK_STATUSBAR(status_bar_block),
			 block_size_status, message);
   }
  g_free(message);
  
  return TRUE;
}

gboolean
affichage_status_bar_timeout (GtkWidget *widget,
			      gchar *action)
{
  gchar *message;
  
  if (!strcmp(action, REFRESH))
    {
      if (timeout >> 0)
	gtk_timeout_remove(timeout);
      clist_main_update();
      timeout = gtk_timeout_add((gint) (1000 * options->time),
				(GtkFunction)clist_main_update, NULL);
      gtk_statusbar_pop(GTK_STATUSBAR(status_bar_timeout),
			timeout_status);
    }
  else if (!strcmp(action, STOP))
    {
      gtk_timeout_remove(timeout);
      gtk_statusbar_pop(GTK_STATUSBAR(status_bar_timeout),
			timeout_status);
      message = g_strdup_printf(_(" Stopped..."));
      gtk_statusbar_push(GTK_STATUSBAR(status_bar_timeout),
			 timeout_status, message);
      g_free(message);
    }
  
  return TRUE;
}

gboolean
on_pop_menu_signal (GtkWidget *clist,
		    GdkEventButton *ev)
{
  if ((ev->button) == 3)
    {
      if (GTK_WIDGET_VISIBLE(popup))
	gtk_menu_popdown(GTK_MENU(popup));
      else
	gtk_menu_popup(GTK_MENU(popup), NULL,
		       NULL, NULL, NULL,
		       ev->button, ev->time);
    }
  
  return TRUE;
}

gboolean
column_number (GtkCList *clist,
	       gint column,
	       gpointer user_data)
{
  gtk_widget_hide(clist_titles[clist->sort_column].arrow[options->sort_type]);
  if (column != clist->sort_column)
    {
      if (column != -1 && column != STATUS_COLUMN &&
	  column != SIZE_COLUMN && column != USED_SPACE_COLUMN &&
	  column != FREE_SPACE_COLUMN && column != CAPACITY_COLUMN)
	{
	  switch (column)
	    {
	    case FILESYSTEM_COLUMN:
	      options->sort_by = FILESYSTEM_COLUMN;
	      break;
	    case MOUNT_POINT_COLUMN:
	      options->sort_by = MOUNT_POINT_COLUMN;
	      break;
	    case USED_PERCENT_COLUMN:
	      options->sort_by = USED_PERCENT_COLUMN;
	      break;
	    case FREE_PERCENT_COLUMN:
	      options->sort_by = FREE_PERCENT_COLUMN;
	      break;
	    case FILESYSTEM_TYPE_COLUMN:
	      options->sort_by = FILESYSTEM_TYPE_COLUMN;
	      break;
	    case MOUNT_OPTIONS_COLUMN:
	      options->sort_by = MOUNT_OPTIONS_COLUMN;
	      break;
	    }
	  gtk_clist_set_sort_column(GTK_CLIST(clist), options->sort_by);
	}
    }
  else
    {
      if (clist->sort_type == GTK_SORT_ASCENDING)
	{
	  options->sort_type = 0;
	  gtk_clist_set_sort_type(GTK_CLIST(clist), GTK_SORT_DESCENDING);
	}
      else
	{
	  options->sort_type = 1;
	  gtk_clist_set_sort_type(GTK_CLIST(clist), GTK_SORT_ASCENDING);
	} 
    }
  gtk_widget_show(clist_titles[options->sort_by].arrow[options->sort_type]);
  gtk_clist_sort(GTK_CLIST(clist));
  
  return TRUE;
}

gboolean
on_selected_row (GtkWidget *list,
		 gint row,
		 gint column,
		 GdkEventButton *event,
		 gpointer data)
{
  selected_line = row;
  
  return TRUE;
}

gboolean
on_unselected_row (GtkWidget *list,
		   gint row,
		   gint column,
		   GdkEventButton *event,
		   gpointer data)
{
  selected_line = -1;
  
  return TRUE;
}
