/*Written with Emacs.*/
/*
  GtkDiskFree shows free space on your mounted partitions.
  Copyright (C) 2001-2002  Dj-Death (Landwerlin Lionel)
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "main.h"
#include "configure.h"

gchar *def_preferences[] = {"window_position",
			    "window_size",
			    "block_size",
			    "virtual_blocks",
			    "not_mounted_blocks",
			    "sort_by",
			    "sort_type",
			    "menubar",
			    "toolbar",
			    "refreshment",
			    "show_all_columns",
			    "hided columns",
			    "color_percent",
			    "degrade",
			    "color_levels",
			    "color1",
			    "color2",
			    NULL};

gchar *preferences_default[] = {"-1x-1",
				"-1x-1",
				"104",
				"0",
				"0",
				"1",
				"1",
				"1",
				"1",
				"2",
				"0",
				"0/0/1/1/1/1/0/0/0/0/1/1",
				"1",
				"1",
				"77/90",
				"0,00000000/0,49999237/0,00000000/",
				"1/0,00000000/0,00000000",
				NULL};

void
print_help (gchar *myname)
{ 
  printf(_("USAGE : %s [OPTION]...\n"), myname);
  printf(_("\n"));
  printf(_("      -d, --default     load gtkdiskfree with default values\n"));
  printf(_("      -t, --time VALUE  look at filesystem every VALUE\n"));
  printf(_("         ex : %s --time 10\n"), myname);
  printf(_("      -h, --help        print this screen\n"));
  printf(_("      -v, --version     print version of gtkdiskfree\n"));
  printf(_("Report any bug to <djdeath@altern.org>.\n"));
  
  return;
}

void
print_error (gchar *myname)
{
  printf(_("Bad option!\n"));
  printf(_("Do `gtkdiskfree --help' to know more.\n"));
  
  return;
}

void
print_version (void)
{
  printf("%s\n", VERSION);
  
  return;
}

void
create_config_directory (void)
{
  gchar *home_dir;
  
  home_dir = g_strconcat(g_get_home_dir(),
			 CONFIG_DIRECTORY,
			 NULL);
  mkdir(home_dir, 0700);
  g_free(home_dir);
  
  return;
}

void
write_config_file_head (FILE *file)
{
  fprintf(file, "# GTKDISKFREE %s config file #\n", VERSION);
  fprintf(file, "# AUTOWRITTEN, DO NOT EDIT #\n\n");
  
  return;
}

void
write_default_config_file (gchar *config_filename)
{
  gint i;
  FILE *file;

  if ((file = fopen(config_filename, "w")) == NULL)
    {
      printf(_("Cannot open %s !\n"), config_filename);
      create_config_directory();
      printf(_("Config directory created.\n"));
      if ((file = fopen(config_filename, "w")) == NULL)
	{
	  printf(_("Please restart gtkdiskfree, it will work next time.\n"));
	  return;
	}
    }

  write_config_file_head(file);
  i = 0;
  while (def_preferences[i] != NULL)
    {
      fprintf(file, "%s = %s\n", def_preferences[i], preferences_default[i]);
      i++;
    }
  fclose(file);
  
  return;
}

void
write_gdouble_board (FILE *file,
		     gint def,
		     gchar *del,
		     gdouble *board,
		     gint num)
{
  GString *line;
  gint i;
  gchar *sign[num];
  
  for (i=0 ; i<num ; i++)
    sign[i] = g_strdup_printf("%f", board[i]);
  
  line = g_string_new("");
  for (i=0 ; i<num ; i++)
    {
      if (i != 0)
	line = g_string_append(line, del);
      line = g_string_append(line, sign[i]);
    }
  
  for (i=0 ; i<num ; i++)
    g_free(sign[i]);
  
  fprintf(file, "%s = %s\n",
	  def_preferences[def],
	  line->str);
  g_string_free(line, TRUE);
  
  return;
}


void
write_gint_board (FILE *file,
		  gint def,
		  gchar *del,
		  gint *board,
		  gint num)
{
  GString *line;
  gint i;
  gchar *sign[num];
  
  for (i=0 ; i<num ; i++)
    sign[i] = g_strdup_printf("%i", board[i]);
  
  line = g_string_new("");
  for (i=0 ; i<num ; i++)
    {
      if (i != 0)
	line = g_string_append(line, del);
      line = g_string_append(line, sign[i]);
    }
  
  for (i=0 ; i<num ; i++)
    g_free(sign[i]);

  fprintf(file, "%s = %s\n",
	  def_preferences[def],
	  line->str);
  g_string_free(line, TRUE);
  
  return;
}

void
read_gdouble_board (gchar *buffer,
		    gdouble *board,
		    gchar *del,
		    gint num)
{
  gint i;
  gchar **sign;
  
  sign = g_strsplit(buffer, del, num);
  
  for (i=0 ; i<num ; i++)
    board[i] = atof(sign[i]);
  g_strfreev(sign);
  
  return;
}

void
read_gint_board (gchar *buffer,
		 gint *board,
		 gchar *del,
		 gint num)
{
  gint i;
  gchar **sign;
  
  sign = g_strsplit(buffer, del, num);
  
  for (i=0 ; i<num ; i++)
    board[i] = atoi(sign[i]);
  g_strfreev(sign);
  
  return;
}

void
switch_read_write_config_file (gint action,
			       gint i,
			       gchar *buffer,
			       FILE *file,
			       preferences_t *pref)
{
  switch (i)
    {
      /*-- Placement de la fenetre --*/
    case (0):
      if (action == READ)
	read_gint_board(buffer, pref->window_position,
			"x", 2);
      else if (action == WRITE)
	{
	  gint x, y;
	  
	  gdk_window_get_position(main_window->window,
				  &x,
				  &y);
	  if ((x * y) != 0)
	    {
	      pref->window_position[0] = x;
	      pref->window_position[1] = y;
	      write_gint_board(file, i, "x",
			       pref->window_position, 2);
	    }
	  else
	    write_gint_board(file, i, "x",
			     pref->window_position, 2);
	}
      else if (action == DEFAULT)
	read_gint_board(preferences_default[0],
			options->window_position, "x", 2);
      break;
      /*-- Taille de la fenetre --*/
    case (1):
      if (action == READ)
	read_gint_board(buffer, pref->window_size,
			"x", 2);
      else if (action == WRITE)
	{
	  gdk_window_get_size(main_window->window,
			      &pref->window_size[0],
			      &pref->window_size[1]);
	  write_gint_board(file, i, "x",
			   pref->window_size, 2);
	}
      else if (action == DEFAULT)
	read_gint_board(preferences_default[1],
			options->window_size, "x", 2);
      break;
      /*-- Taille de blocks --*/
    case (2):
      if (action == READ)
	pref->blocks = atoi(buffer);
      else if (action == WRITE)
	fprintf(file, "%s = %i\n",
		def_preferences[i],
		pref->blocks);
      else if (action == DEFAULT)
	options->blocks = atoi(preferences_default[2]);
      break;
      /*-- Tous les blocks --*/
    case (3):
      if (action == READ)
	pref->virtual_blocks = atoi(buffer);
      else if (action == WRITE)
	fprintf(file, "%s = %i\n",
		def_preferences[i],
		pref->virtual_blocks);
      else if (action == DEFAULT)
	options->virtual_blocks = atoi(preferences_default[3]);
      break;
      /*-- Autres blocks --*/
   case (4):
      if (action == READ)
	pref->not_mounted_blocks = atoi(buffer);
      else if (action == WRITE)
	fprintf(file, "%s = %i\n",
		def_preferences[i],
		pref->not_mounted_blocks);
      else if (action == DEFAULT)
	options->not_mounted_blocks = atoi(preferences_default[4]);
       break;
       /*-- Tri de la colonne --*/
    case (5):
      if (action == READ)
	pref->sort_by = atoi(buffer);
      else if (action == WRITE)
       	fprintf(file, "%s = %i\n",
		def_preferences[i],
		pref->sort_by);
      else if (action == DEFAULT)
	options->sort_by = atoi(preferences_default[5]);
      break;
      /*-- Tri ascendant/descendant --*/
    case (6):
      if (action == READ)
	pref->sort_type = atoi(buffer);
      else if (action == WRITE)
	fprintf(file, "%s = %i\n",
		def_preferences[i],
		pref->sort_type);
      else if (action == DEFAULT)
	options->sort_type = atoi(preferences_default[6]);
      break;
      /*-- Barre de menu --*/
    case (7):
      if (action == READ)
	pref->menubar = atoi(buffer);
      else if (action == WRITE)
	fprintf(file, "%s = %i\n",
		def_preferences[i],
		pref->menubar);
      else if (action == DEFAULT)
	options->menubar = atoi(preferences_default[7]);
      break;
      /*-- Barre d'icone --*/
    case (8):
      if (action == READ)
	pref->toolbar = atoi(buffer);
      else if (action == WRITE)
	fprintf(file, "%s = %i\n",
		def_preferences[i],
		pref->toolbar);
      else if (action == DEFAULT)
	options->toolbar = atoi(preferences_default[8]);
      break;
      /*-- Temps de rafrachissement --*/
    case (9):
      if (action == READ)
	pref->time = atof(buffer);
      else if (action == WRITE)
	fprintf(file, "%s = %f\n",
		def_preferences[i],
		pref->time);
      else if (action == DEFAULT)
	options->time = atof(preferences_default[9]);
      break;
      /*-- Toutes les colonnes visibles --*/
    case (10):
      if (action == READ)
	pref->show_all_columns = atoi(buffer);
      else if (action == WRITE)
	fprintf(file, "%s = %i\n",
		def_preferences[i],
		pref->show_all_columns);
      else if (action == DEFAULT)
	options->show_all_columns = atoi(preferences_default[10]);
      break;
      /*-- Colonnes visibles --*/
    case (11):
      if (action == READ)
	read_gint_board(buffer, pref->hided_columns, "/", NBRE_COLONNES);
      else if (action == WRITE)
	write_gint_board(file, i, "/", pref->hided_columns, NBRE_COLONNES);
      else if (action == DEFAULT)
	read_gint_board(preferences_default[11],
			options->hided_columns, "/", NBRE_COLONNES);
      break;
      /*-- Affichage de pourcentages dans la jauge --*/
    case (12):
      if (action == READ)
	pref->color_percent = atoi(buffer);
      else if (action == WRITE)
	fprintf(file, "%s = %i\n",
		def_preferences[i],
		pref->color_percent);
      else if (action == DEFAULT)
	options->color_percent = atoi(preferences_default[12]);
      break;
      /*-- Dgrad --*/
    case (13):
      if (action == READ)
	pref->degrade = atoi(buffer);
      else if (action == WRITE)
	fprintf(file, "%s = %i\n",
		def_preferences[i],
		pref->degrade);
      else if (action == DEFAULT)
	options->degrade = atoi(preferences_default[13]);
      break;
      /*-- Niveau des couleurs --*/
    case (14):
      if (action == READ)
	read_gint_board(buffer, pref->color_levels, "/", 2);
      else if (action == WRITE)
	write_gint_board(file, i, "/", pref->color_levels, 2);
      else if (action == DEFAULT)
	read_gint_board(preferences_default[14],
			options->color_levels, "/", 2);
      break;
      /*-- 1 color --*/
    case (15):
      if (action == READ)
	read_gdouble_board(buffer, pref->color1, "/", 3);
      else if (action == WRITE)
	write_gdouble_board(file, i, "/", pref->color1, 3);
      else if (action == DEFAULT)
	read_gdouble_board(preferences_default[16],
			   options->color1, "/", 3);
      break;
    case (16):
      if (action == READ)
	read_gdouble_board(buffer, pref->color2, "/", 3);
      else if (action == WRITE)
	write_gdouble_board(file, i, "/", pref->color2, 3);
      else if (action == DEFAULT)
	read_gdouble_board(preferences_default[16],
			   options->color2, "/", 3);
      break;
    }
  return;
}

void
load_config_file_values (void)
{
  gint i, j;
  gchar *config_filename, *buffer = NULL, **lines = NULL;
  FILE *file;
  struct stat stats;
  
  config_filename = g_strconcat(g_get_home_dir(),
				CONFIG_DIRECTORY,
				CONFIG_FILENAME,
				NULL);

  if ((file = fopen(config_filename, "r")) == NULL)
    {
      write_default_config_file(config_filename);
      if ((file = fopen(config_filename, "r")) == NULL)
	{
	  g_free(config_filename);
	  return;
	}
    }
  if (lstat(config_filename, &stats) == -1)
    {
      fclose(file);
      g_free(config_filename);
      return;
    }
  buffer = g_malloc(stats.st_size + 1);
  if (fread(buffer, stats.st_size, 1, file) == 0)
    {
      g_free(config_filename);
      g_free(buffer);
      fclose(file);
      return;
    }
  g_free(config_filename);
  buffer[stats.st_size] = '\0';
  lines = g_strsplit(buffer, "\n", 0);
  g_free(buffer);
  
  i = 0;
  while (def_preferences[i] != NULL)
    {
      gchar *option = NULL;
      gchar **escape;
      
      j = 0;
      while (lines[j])
	{
	  if (strstr(lines[j], "#") != NULL)
	    {
	      escape = g_strsplit(lines[j], "#", 1);
	      if (strstr(escape[0], def_preferences[i]) != NULL)
		{
		  gchar **words;
		  
		  words = g_strsplit(escape[0], "=", 0);
		  option = g_strdup(words[1]);
		  g_strfreev(words);
		  break;
		}
	      g_strfreev(escape);
	    }
	  else
	    {
	      if (strstr(lines[j], def_preferences[i]) != NULL)
		{
		  gchar **words;
		  
		  words = g_strsplit(lines[j], "=", 0);
		  option = g_strdup(words[1]);
		  g_strfreev(words);
		  break;
		}
	    }
	  j++;
	}
      if (option == NULL)
	switch_read_write_config_file(DEFAULT, i, option, NULL, options);
      else
	{
	  switch_read_write_config_file(READ, i, option, NULL, options);
	  g_free(option);
	}
      fseek(file, 0L, SEEK_SET);
      i++;
    }
  g_strfreev(lines);
  fclose(file);
  
  return;
}
		 
void
write_config_file (preferences_t *pref)
{
  gint i;
  gchar *config_filename;
  FILE *file;
  
  config_filename = g_strconcat(g_get_home_dir(),
				CONFIG_DIRECTORY,
				CONFIG_FILENAME,
				NULL);
  
  
  if ((file = fopen(config_filename, "w")) == NULL)
    {
      printf(_("Cannot open %s !\n"), config_filename);
      create_config_directory();
      printf(_("Config directory created.\n"));
      if ((file = fopen(config_filename, "w")) == NULL)
	{
	  printf(_("Please restart gtkdiskfree, it will work next time.\n"));
	  return;
	}
    }
  g_free(config_filename);
  
  write_config_file_head(file);
  
  i = 0;
  while (def_preferences[i] != NULL)
    {
      switch_read_write_config_file(WRITE, i, NULL, file, pref);
      i++;
    }
  
  fclose(file);
  
  return;
}

void
options_init (gint argc, gchar **argv)
{
  gint time, version, help, defaut, c;
  struct option long_options[] =
  {
    {"default", no_argument, &defaut, 'd'},
    {"help", no_argument, &help, 'h'},
    {"time", required_argument, &time, 't'},
    {"version", no_argument, &version, 'v'},
    {NULL, 0, NULL, 0}
  };
  
  time = 0;
  version = 0;
  help = 0;
  defaut = 0;
  
  options = g_malloc(sizeof(preferences_t));
  
  while ((c = getopt_long(argc, argv, "dht:v", long_options, NULL)) != -1)
    {
      switch (c)
	{
	case 0:
	  break;
	case 'd':
	  defaut = 1;
	  break;
	case 'h':
	  help = 1;
	  break;
	case 't':
	  break;
	case 'v':
	  version = 1;
	  break;
	default:
	  print_help(argv[0]);
	  exit(0);
	}
    }  
  if (help)
    {
      print_help(argv[0]);
      exit(0);
    }
  if (version)
    {
      print_version();
      exit(0);
    }
  if (defaut)
    load_defaults_values();
  else
    load_config_file_values();
  if (time != 0)
    options->time = time;
  
  return;
}

void
load_defaults_values (void)
{
  gint i = 0;
  
  while (def_preferences[i] != NULL)
    {
      switch_read_write_config_file(DEFAULT, i, NULL,
				    NULL, options);
      i++;
    }
  
  return;
}
