/*Written with Emacs.*/
/*
  GtkDiskFree shows free space on your mounted partitions.
  Copyright (C) 2001-2002  Dj-Death (Landwerlin Lionel)
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

#include "main.h"
#include "diskfree.h"

#include "icons/bar.xpm"

void
clist_set_lines_removable (void)
{
  gint line;
  
  line = 0;
  while (!clist_test_case_vide(line))
    {
      gtk_clist_set_text(GTK_CLIST(clist), line, STATUS_COLUMN, OLD); 
      line++;
    }
  
  return;
}

void
clist_set_line_unremovable (gint line)
{
  gtk_clist_set_text(GTK_CLIST(clist), line, STATUS_COLUMN, NEW);
  
  return;
}

void
clist_remove_old_lines (void)
{
  gint line;
  
  line = 0;
  while (!clist_test_case_vide(line))
    {
      if (clist_test_case_texte(line, STATUS_COLUMN, OLD))
 	clist_update(REMOVE, line, NULL);
      else
 	line++;
    }
  
  return;
}

gboolean
clist_test_case_vide (gint line)
{
  guint ans;
  gchar *text_line;

  ans = gtk_clist_get_text(GTK_CLIST(clist), line, STATUS_COLUMN,  &text_line);
  if (ans)
    return FALSE;
  else
    return TRUE;
}

gboolean
clist_test_case_texte (gint line,
		       gint colonne,
		       gchar *text)
{
  gchar *text_line;
  
  gtk_clist_get_text(GTK_CLIST(clist), line, colonne,  &text_line);

  if (!strcmp(text_line, text))
    return TRUE;
  else
    return FALSE;
}

gint
clist_find_line (gchar *mount_point)
{
  gint i;

  i = 0;
  while (!clist_test_case_vide(i))
    {
      if (clist_test_case_texte(i, MOUNT_POINT_COLUMN, mount_point))
	break;
      i++;
    }
  if (clist_test_case_vide(i))
    return -1;
  else
    return i;
}

/*
  Line needs to be updated ?
*/
gboolean
clist_line_needs_update (gint line, gchar **sys)
{
  gint colonne;
  
  if (color_update)
    return TRUE;
  
  for (colonne=1 ; colonne<NBRE_COLONNES-1 ; colonne++)
    {
      if (!clist_test_case_texte(line, colonne, sys[colonne]))
	return TRUE;
    }
  
  return FALSE;
}


void
clist_make_pixmap (gint line,
		   gint percent)
{
  GdkPixmap *pixmap;
  GdkBitmap *bitmap;
  GdkFont *font;
  GdkColor trans, color;
  GdkColormap *palette;
  GdkGC *gc;
  
  pixmap = gdk_pixmap_create_from_xpm_d(main_window->window,
					&bitmap,
					&trans,
					bar_xpm);
  
  palette = gdk_window_get_colormap(main_window->window);

  gc = gdk_gc_new(pixmap);
  gdk_gc_set_line_attributes(gc, 1,
			     GDK_LINE_SOLID,
			     GDK_CAP_BUTT,
			     GDK_JOIN_MITER);
  
  /*
    Backgound color : white.
  */
  color.red = 65535;
  color.green = 65535;
  color.blue = 65535;
  gdk_color_alloc(palette, &color);
  gdk_gc_set_background(gc, &color);
  /*
    Foreground color.
  */
  if (percent == -1)
    {
      color.red = 49150;
      color.green = 49150;
      color.blue = 49150;
      gdk_color_alloc(palette, &color);
      gdk_gc_set_foreground(gc, &color);
      gdk_draw_rectangle(pixmap, gc, TRUE, 2, 3, 100, 11);
    }
  else if (options->degrade)
    {
      GdkColor color2;
      gint i;
      div_t red, green, blue;
      
      color.red = (gint) (65535 * options->color1[0]);
      color.green = (gint) (65535 * options->color1[1]);
      color.blue = (gint) (65535 * options->color1[2]);
      
      color2.red = (gint) (65535 * options->color2[0]);
      color2.green = (gint) (65535 * options->color2[1]);
      color2.blue = (gint) (65535 * options->color2[2]);
      
      red = div((color.red - color2.red), 100);
      green = div((color.green - color2.green), 100);
      blue = div((color.blue - color2.blue), 100);
      gdk_color_alloc(palette, &color);
      gdk_gc_set_foreground(gc, &color);
      for (i=2 ; i<percent+1 ; i++)
	{
	  gdk_draw_line(pixmap, gc, i, 3, i, 14);
	  color.red = color.red - red.quot;
	  color.green = color.green - green.quot;
	  color.blue = color.blue - blue.quot;
	  gdk_color_alloc(palette, &color);
	  gdk_gc_set_foreground(gc, &color);
	}
    }
  else 
    {
      if (percent <= options->color_levels[0]) 
	{
	  color.red = 0;
	  color.green = 32767;
	  color.blue = 0;
	}
      else if (percent <= options->color_levels[1]) 
	{
	  color.red = 65535;
	  color.green = 32767; color.blue = 0;
	}
      else 
	{
	  color.red = 65535;
	  color.green = 0;
	  color.blue = 0;
	}
      gdk_color_alloc(palette, &color);
      gdk_gc_set_foreground(gc, &color);
      gdk_draw_rectangle(pixmap, gc, TRUE, 2, 3, percent, 11);
    }
    
  /*
    Add text in capacity column ?
   */
  if (options->color_percent)
    {
      gchar *text;
      
      if (percent == -1)
	text = g_strdup(" - ");
      else
	text = g_strdup_printf("%i%%", percent);
      font = GTK_WIDGET(clist)->style->font;
      color.red = 0;
      color.green = 0;
      color.blue = 0;
      gdk_color_alloc(palette, &color);
      gdk_gc_set_foreground(gc, &color);
      gdk_draw_string(pixmap, font, gc,
		       46, 13, text);
      g_free(text);
    }
  
  gtk_clist_set_pixmap(GTK_CLIST(clist), line,
		       CAPACITY_COLUMN,
		       pixmap, bitmap);
  gdk_pixmap_unref(pixmap);
  gdk_pixmap_unref(bitmap);
  gdk_gc_destroy(gc);
  
  return;
}

/* 
   Just update.
*/
void
clist_update (gint action,
	      gint line,
	      gchar **sys)
{
  gint new_line;
  gint colonne;
  
  gtk_clist_freeze(GTK_CLIST(clist));
  
  switch(action)
    {
    case(ADD):
      new_line = gtk_clist_append(GTK_CLIST(clist), sys);
      if (!strcmp(sys[USED_PERCENT_COLUMN], "-"))
	clist_make_pixmap(new_line, -1);
      else
	clist_make_pixmap(new_line, atoi(sys[USED_PERCENT_COLUMN]));
      break;
    case(INSERT):
      gtk_clist_insert(GTK_CLIST(clist), line, sys);
      if (!strcmp(sys[USED_PERCENT_COLUMN], "-"))
	clist_make_pixmap(line, -1);
      else
	clist_make_pixmap(line, atoi(sys[USED_PERCENT_COLUMN]));
      break;  
    case(UPDATE):
      for (colonne=0 ; colonne<NBRE_COLONNES-1 ; colonne++)
	gtk_clist_set_text(GTK_CLIST(clist), line, colonne, sys[colonne]);
      if (!strcmp(sys[USED_PERCENT_COLUMN], "-"))
	clist_make_pixmap(line, -1);
      else
	clist_make_pixmap(line, atoi(sys[USED_PERCENT_COLUMN]));
      break;
    case(REMOVE):
      gtk_clist_remove(GTK_CLIST(clist), line);
      break;
    }
  gtk_clist_thaw(GTK_CLIST(clist));
  
  return;
}

/*
  This function check if the line
  (from the filesystem) needs to
  be displayed. According to
  options->not_mounted_blocks and
  options->virtual_blocks.
*/
void
clist_pre_update (filesystem_t *sys)
{
  if (!sys->mounted && !options->not_mounted_blocks)
    return;
  
  if (!sys->real && !options->virtual_blocks)
    return;

  if (sys->line != -1)
    {
      if (clist_line_needs_update(sys->line, sys->file))
	clist_update(UPDATE, sys->line, sys->file);
      else
	clist_set_line_unremovable(sys->line);
    }
  else
    clist_update(ADD, 0, sys->file);
  
  return;
}

/*
  Return the percent.
*/
gchar *
list_filesystem_percent (glong all,
			 glong value)
{
  gchar *percent;
  ldiv_t divis;
  
  divis = ldiv((value*100), all);
  percent = g_strdup_printf("%i%%", (gint) divis.quot);
  
  return percent;
}

/*
  This function put values in
  the requested user format.
  According to options->blocks.
*/
gchar *
list_filesystem_size (glong blocks,
		      glong value)
{
  gchar *size, *pre_size;
  ldiv_t pour, pour2;
  
  pour = ldiv(blocks, 1024);
  pre_size = g_strdup_printf("%ld.%ld", pour.quot, pour.rem);
  if (strstr(pre_size, "0.") != NULL)
    {
      g_free(pre_size);
      pre_size = g_strdup_printf("%ld", pour.rem * value);
      pour = ldiv(atol(pre_size), 1000);
      size = g_strdup_printf("%ld", pour.quot);
      g_free(pre_size);
    }
  else
    {
      size = g_strdup_printf("%i", (gint) (atof(pre_size) * (gdouble) value));
      g_free(pre_size);
    }     
  
  if (!atoi(size))
    {
      g_free(size);
      size = g_strdup("0");
      
      return size;
    }
  
  switch (options->blocks)
    {
    case 'k':
      break;
    case 'm':
      pour = ldiv(atol(size), 1024);
      pour2 = ldiv(pour.rem, 100);
      g_free(size);
      if (pour2.quot == 0)
	size = g_strdup_printf("%ld", pour.quot);
      else
	size = g_strdup_printf("%ld.%ld", pour.quot, pour2.quot);
      break;
    case 'g':
      pour = ldiv(atol(size), 1024*1024);
      pour2 = ldiv(pour.rem, 10000);
      g_free(size);
      size = g_strdup_printf("%ld.%ld", pour.quot, pour2.quot);
      break;
    case 'h':
      pour = ldiv(atol(size), 1024*1024*1024);
      if (pour.quot != 0)
	{
	  pour2 = ldiv(pour.rem, 10000);
	  g_free(size);
	  size = g_strdup_printf("%ld.%ld T", pour.quot, pour2.quot);
	}
      else
	{
	  pour = ldiv(atol(size), 1024*1024);
	  if (pour.quot != 0)
	    {
	      pour2 = ldiv(pour.rem, 10000);
	      g_free(size);
	      size = g_strdup_printf("%ld.%ld G", pour.quot, pour2.quot);
	    }
	  else
	    {
	      pour = ldiv(atol(size), 1024);
	      if (pour.quot != 0)
		{
		  pour2 = ldiv(pour.rem, 100);
		  g_free(size);
		  if (pour2.rem == 0)
		    size = g_strdup_printf("%ld M", pour.quot);
		  else
		    size = g_strdup_printf("%ld.%ld M", pour.quot, pour2.quot);
		}
	      else
		{
		  pre_size = g_strdup(size);
		  g_free(size);
		  size = g_strdup_printf("%s K", pre_size);
		  g_free(pre_size);
		}
	    }
	}
      break;
    }
  
  return size;
}

void
list_filesystem_info (gchar *mount_point,
		      filesystem_t *sys)
{
  struct statfs sts;
  
  if ((statfs(mount_point, &sts) == -1) || (!sts.f_blocks))
    {
      sys->real = FALSE;
      sys->file[SIZE_COLUMN] = g_strdup("0");
      sys->file[USED_SPACE_COLUMN] = g_strdup("0");
      sys->file[FREE_SPACE_COLUMN] = g_strdup("0");
      sys->file[USED_PERCENT_COLUMN] = g_strdup("-");
      sys->file[FREE_PERCENT_COLUMN] = g_strdup("-");
    }
  else
    {
      sys->real = TRUE;
      sys->file[SIZE_COLUMN] = list_filesystem_size(sts.f_bsize,
						    sts.f_blocks);
      sys->file[USED_SPACE_COLUMN] = list_filesystem_size(sts.f_bsize,
							  (sts.f_blocks - sts.f_bfree));
      sys->file[FREE_SPACE_COLUMN] = list_filesystem_size(sts.f_bsize,
							  sts.f_bfree);
      sys->file[USED_PERCENT_COLUMN] = list_filesystem_percent(sts.f_blocks,
							       (sts.f_blocks - sts.f_bfree));
      sys->file[FREE_PERCENT_COLUMN] = list_filesystem_percent(sts.f_blocks,
							       sts.f_bfree);
    }
  
  return;
}

void
list_not_mounted_filesystem (struct mntent *mnt,
			     filesystem_t *sys)
{
  sys->mounted = 0;
  sys->real = 1;
  sys->file[STATUS_COLUMN] = g_strdup(NEW);
  sys->file[STATE_COLUMN] = g_strdup(UMOUNT);
  sys->file[FILESYSTEM_COLUMN] = g_strdup(mnt->mnt_fsname);
  sys->file[FILESYSTEM_TYPE_COLUMN] = g_strdup(mnt->mnt_type);
  sys->file[MOUNT_OPTIONS_COLUMN] = g_strdup(mnt->mnt_opts);
  sys->file[SIZE_COLUMN] = g_strdup("0");
  sys->file[USED_SPACE_COLUMN] = g_strdup("0");
  sys->file[FREE_SPACE_COLUMN] = g_strdup("0");
  sys->file[USED_PERCENT_COLUMN] = g_strdup("-");
  sys->file[FREE_PERCENT_COLUMN] = g_strdup("-");  
  
  return;
}

void
list_mounted_filesystem (struct mntent *mnt,
			 filesystem_t *sys)
{
  /*** Le filesystem est mont. ***/
  sys->mounted = 1;
  sys->file[STATUS_COLUMN] = g_strdup(NEW);
  sys->file[STATE_COLUMN] = g_strdup(MOUNT);
  sys->file[FILESYSTEM_COLUMN] = g_strdup(mnt->mnt_fsname);
  sys->file[FILESYSTEM_TYPE_COLUMN] = g_strdup(mnt->mnt_type);
  sys->file[MOUNT_OPTIONS_COLUMN] = g_strdup(mnt->mnt_opts);
  list_filesystem_info(mnt->mnt_dir, sys);
  
  return;
}

gboolean
list_search (filesystem_t *sys,
	     gchar *text)
{
  if (!strcmp(sys->file[MOUNT_POINT_COLUMN], text))
    return TRUE;
  else
    return FALSE;
}

void
list_free (filesystem_t *sys)
{
  gint i;
  
  for (i=0 ; i<NBRE_COLONNES ; i++)
    g_free(sys->file[i]);
  g_free(sys);
  
  return;
}

void
list_filesystems (void)
{
  GList *list = NULL, *sear = NULL;
  FILE *file;
  struct mntent *mnt;
  
  /*
    We get all filesystems, with the /etc/fstab
    file, and we put it in the list. We just
    put the mount point as information.
  */
  file = setmntent(FSTAB_FILESYSTEMS, "r");
  
  while ((mnt = getmntent(file)) != NULL)
    {
      if (strcmp(mnt->mnt_dir, "swap") != 0)
	{
	  filesystem_t *fil;
	  
	  fil = g_malloc(sizeof(filesystem_t));
	  fil->line = clist_find_line(mnt->mnt_dir);
	  fil->mounted = 0;
	  fil->file[MOUNT_POINT_COLUMN] = g_strdup(mnt->mnt_dir);
	  fil->file[CAPACITY_COLUMN] = g_strdup("");
	
	  list = g_list_append(list, fil);
	}
    } 
  endmntent(file);
  free(mnt);
  
  /*
    We get now the mounted filesystems with
    the /etc/mtab file. For each we see if
    there is in list. If yes, we put theirs
    values,if not we add the filesystem and
    we put theirs values.
  */
  file = setmntent(MTAB_FILESYSTEMS, "r");
  
  while ((mnt = getmntent(file)) != NULL)
    {
      list = g_list_first(list);
      sear = list;
      while (sear)
	{
	  if (list_search(((filesystem_t *)sear->data), mnt->mnt_dir))
	    {
	      list_mounted_filesystem(mnt, ((filesystem_t *)(sear->data)));
	      break;
	    }
	  sear = sear->next;
	}
      if (sear == NULL)
	{
	  filesystem_t *fil;
	  
	  fil = g_malloc(sizeof(filesystem_t));
	  fil->line = clist_find_line(mnt->mnt_dir);
	  fil->file[MOUNT_POINT_COLUMN] = g_strdup(mnt->mnt_dir);
	  fil->file[CAPACITY_COLUMN] = g_strdup("");
	  list_mounted_filesystem(mnt, fil);
	  list = g_list_append(list, fil);
	}
      sear = NULL;
    }
  endmntent(file);
  free(mnt);
  
  /*
    We search all not mounted filesystems
    in the list, and we put values of not
    mounted filesystem 
    (see list_not_mounted_filesystem function)
    in the list.
  */
  file = setmntent(FSTAB_FILESYSTEMS, "r");
  
  while ((mnt = getmntent(file)) != NULL)
    {
      list = g_list_first(list);
      sear = list;
      
      while (sear)
	{
	  if (list_search(((filesystem_t *)sear->data), mnt->mnt_dir) &&
	      !((filesystem_t *)sear->data)->mounted)
	    {
	      list_not_mounted_filesystem(mnt, (filesystem_t *)sear->data);
	      break;
	    }
	  sear = sear->next;
	}
      sear = NULL;
    }
  endmntent(file);
  free(mnt);
  
  /*
    We set the STATUS case of all lines to
    0, the if a line isn't update by putting
    this value to 1, she is removed.
  */
  clist_set_lines_removable();
  /*
    We fill the clist.
  */
  g_list_foreach(list, (GFunc)clist_pre_update, NULL);
  clist_remove_old_lines();
  
  /*
    Finaly we free the memory allocated by the list.
  */
  g_list_foreach(list, (GFunc)list_free, NULL);
  g_list_free(list);
  
  return;
}

gboolean
clist_main_update (void)
{
  list_filesystems();
  
  color_update = 0;
  
  return TRUE;
}
