/*Written with Emacs.*/
/*
  GtkDiskFree shows free space on your mounted partitions.
  Copyright (C) 2001-2002  Dj-Death (Landwerlin Lionel)
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "main.h"
#include "callbacks.h"
#include "interface.h"
#include "menus.h"

#include "icons/atr_window.xpm"

void
init_sort_way (void)
{
  if (options->sort_type == 1)
    gtk_clist_set_sort_type(GTK_CLIST(clist), GTK_SORT_ASCENDING);
  else
    gtk_clist_set_sort_type(GTK_CLIST(clist), GTK_SORT_DESCENDING);
  gtk_clist_set_sort_column(GTK_CLIST(clist), options->sort_by);
  gtk_widget_show(clist_titles[options->sort_by].arrow[options->sort_type]);
  gtk_clist_sort(GTK_CLIST(clist));
  
  return;
}

void
set_main_window_position_size (void)
{
  if (options->window_size[0] != -1 &&
      options->window_size[1] != -1)
    gtk_window_set_default_size(GTK_WINDOW(main_window),
				options->window_size[0],
				options->window_size[1]);
  
  if (gdk_screen_width() >= options->window_position[0] &&
      gdk_screen_height() >= options->window_position[1] &&
      (options->window_position[0] != -1 && options->window_position[1] != -1))
    {
      gtk_window_reposition(GTK_WINDOW(main_window),
			    options->window_position[0],
			    options->window_position[1]);
    }
  gdk_window_set_decorations(main_window->window, GDK_DECOR_ALL);
  
  return;
}

void
show_hide_columns (void)
{
  gint i;

  gtk_clist_set_column_visibility(GTK_CLIST(clist), 0, FALSE);
  gtk_clist_set_column_visibility(GTK_CLIST(clist), 1, FALSE);
  if (options->show_all_columns)
    for (i=2 ; i<NBRE_COLONNES ; i++)
      gtk_clist_set_column_visibility(GTK_CLIST(clist), i, TRUE);
  else
    {
      for (i=2 ; i<NBRE_COLONNES ; i++)
	{
	  if (options->hided_columns[i])
	    gtk_clist_set_column_visibility(GTK_CLIST(clist), i, TRUE);
	  else
	    gtk_clist_set_column_visibility(GTK_CLIST(clist), i, FALSE);
	}
    }
  
  return;
}

void
main_gtk_window (void)
{
  gint i;
  gchar *titles[NBRE_COLONNES] = {_("status"),
				  _("state"),
				  _("Filesystem"),
				  _("Size"),
				  _("Used space"),
				  _("Free space"),
				  _("Used %"),
				  _("Free %"),
				  _("Filesystem type"),
				  _("Mount option's"),
				  _("Mounted on"),
				  _("Capacity")};
  
  GdkPixmap *icon_pixmap;
  GdkBitmap *icon_mask;

  GtkWidget *main_box, *menu_box;
  GtkWidget *src;

  GtkWidget *status_hbox, *status_table, *status_bar;
  
  /*--MainWindow--*/
  
  main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  gtk_window_set_title(GTK_WINDOW(main_window), "gtkdiskfree");
  gtk_signal_connect(GTK_OBJECT(main_window), "delete_event",
		     GTK_SIGNAL_FUNC(main_quit), NULL);
  gtk_widget_realize(main_window);
  set_main_window_position_size();
  gtk_window_set_policy(GTK_WINDOW(main_window),
			TRUE, TRUE, TRUE);
  icon_pixmap = gdk_pixmap_create_from_xpm_d(main_window->window,
					     &icon_mask,
					     &main_window->style->bg[GTK_STATE_NORMAL],
					     atr_window_xpm);
  gdk_window_set_icon(main_window->window, NULL, icon_pixmap, icon_mask);
  gdk_window_set_icon_name(main_window->window, "GtkDiskFree");
    
  main_box = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(main_window), main_box);
  gtk_widget_show(main_box);
  
  /*--MenuBar & ToolBar--*/
  
  menu_box = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(main_box), menu_box, FALSE, FALSE, 0);
    
  main_menu_create(menu_box);
  toolbar_create(main_window, menu_box);
  show_hide_menus();
  gtk_widget_show(menu_box);
  
  /*--Clist--*/ 
  
  clist_box = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(main_box), clist_box, TRUE, TRUE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(clist_box), 5);
  gtk_widget_show(clist_box);

  src = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(src),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start(GTK_BOX(clist_box), src, TRUE, TRUE, 0);
  gtk_widget_show(src);
    
  clist = gtk_clist_new(NBRE_COLONNES);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(src), clist);
  gtk_widget_show(clist);
  
  for (i=0 ; i<NBRE_COLONNES ; i++)
    {
      clist_titles[i].label = gtk_label_new(titles[i]);
      clist_titles[i].hbox = gtk_hbox_new(FALSE, 0);
      clist_titles[i].arrow[0] = gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_IN);
      clist_titles[i].arrow[1] = gtk_arrow_new(GTK_ARROW_UP, GTK_SHADOW_IN);
      gtk_clist_set_column_auto_resize(GTK_CLIST(clist), i, TRUE);
      gtk_box_pack_start(GTK_BOX(clist_titles[i].hbox),
			 clist_titles[i].label,
			 FALSE, FALSE, 0);
      gtk_box_pack_end(GTK_BOX(clist_titles[i].hbox),
		       clist_titles[i].arrow[0],
		       FALSE, FALSE, 0);
      gtk_box_pack_end(GTK_BOX(clist_titles[i].hbox),
		       clist_titles[i].arrow[1],
		       FALSE, FALSE, 0);
      gtk_clist_set_column_widget(GTK_CLIST(clist),
				  i, clist_titles[i].hbox);
      gtk_widget_show(clist_titles[i].label);
      gtk_widget_show(clist_titles[i].hbox);
    }
  gtk_clist_column_titles_show(GTK_CLIST(clist));
  gtk_clist_column_titles_active(GTK_CLIST(clist));
  gtk_clist_set_reorderable(GTK_CLIST(clist), TRUE);
  gtk_signal_connect(GTK_OBJECT(clist), "select_row",
		     GTK_SIGNAL_FUNC(on_selected_row),
		     NULL);
  gtk_signal_connect(GTK_OBJECT(clist), "unselect_row",
		     GTK_SIGNAL_FUNC(on_unselected_row),
		     NULL);
  gtk_signal_connect(GTK_OBJECT(clist), "button-release-event",
		     GTK_SIGNAL_FUNC(on_pop_menu_signal),
		     NULL);
  gtk_signal_connect(GTK_OBJECT(clist), "click-column",
		     GTK_SIGNAL_FUNC(column_number),
		     NULL);
  selected_line = -1;
  show_hide_columns();
  
 /*--StatusBar--*/

  status_hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(main_box), status_hbox, FALSE, FALSE, 0);
  gtk_widget_show(status_hbox);

  status_table = gtk_table_new(1, 5, TRUE);
  gtk_box_pack_start(GTK_BOX(status_hbox), status_table, TRUE, TRUE, 0);
  gtk_widget_show(status_table);
  
  status_bar = gtk_statusbar_new();
  gtk_table_attach( GTK_TABLE(status_table), status_bar, 0, 3, 0, 1,
		    GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
  gtk_widget_show(status_bar);
  
  status_bar_timeout = gtk_statusbar_new();
  gtk_table_attach( GTK_TABLE(status_table), status_bar_timeout, 3, 4, 0, 1,
		    GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
  gtk_widget_show(status_bar_timeout);
  
  timeout_status = gtk_statusbar_get_context_id(GTK_STATUSBAR(status_bar_timeout),
						"Program status");
  status_bar_block = gtk_statusbar_new();
  gtk_table_attach( GTK_TABLE(status_table), status_bar_block, 4, 5, 0, 1,
		    GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
  gtk_widget_show(status_bar_block);
  
  block_size_status = gtk_statusbar_get_context_id(GTK_STATUSBAR(status_bar_block),
						   "Blocks size status");
  gtk_widget_show(status_hbox);
  affichage_status_bar_block_size();

  /*-------*/
  gtk_widget_show(main_window);
   
  return;
}
