/*Written with Emacs.*/
/*
  GtkDiskFree shows free space on your mounted partitions.
  Copyright (C) 2001-2002  Dj-Death (Landwerlin Lionel)
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "main.h"
#include "menus.h"
#include "callbacks.h"
#include "options.h"
#include "mount.h"
#include "about.h"
#include "widgets.h"

#include "icons/mount.xpm"
#include "icons/umount.xpm"
#include "icons/refresh.xpm"
#include "icons/stop.xpm"
#include "icons/options.xpm"
#include "icons/help.xpm"

gboolean
menu_signal_handle(GtkWidget *widget,
		   GtkWidget *child,
		   gpointer data)
{
  if (clist_box != NULL)
    {
      gtk_widget_hide(clist_box);
      gtk_widget_show(clist_box);
    }
  
  return TRUE;
}

void
show_hide_menus ()
{
  if (options->menubar)
    gtk_widget_show(menu_handle_box);
  else 
    gtk_widget_hide(menu_handle_box);
  
  if (options->toolbar)
    gtk_widget_show(tool_handle_box);
  else
    gtk_widget_hide(tool_handle_box);
  menu_signal_handle(NULL, NULL, NULL);
  
  return;
}

void
main_menu_create (GtkWidget *box)
{
  GtkWidget *menubar;
  GtkWidget *main_menu;
  GtkWidget *main_item;
  GtkWidget *menu_item;

  /* -- Create menu bar -- */

  menu_handle_box = gtk_handle_box_new();
  gtk_box_pack_start(GTK_BOX(box), menu_handle_box, FALSE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(menu_handle_box), "child_detached",
		     GTK_SIGNAL_FUNC(menu_signal_handle), NULL);
  
  menubar = gtk_menu_bar_new();
  gtk_widget_set_usize(menubar, -1, 25);
  gtk_container_add(GTK_CONTAINER(menu_handle_box), menubar);
  gtk_widget_show(menubar);
 
  /* -- File menu -- */
  
  main_item = gtk_menu_item_new_with_label(_("File"));
  gtk_menu_bar_append(GTK_MENU_BAR(menubar), main_item);
  gtk_widget_show(main_item);
  
  main_menu = add_menu(main_item);
  
  menu_item = add_menu_item(_("Mount"), main_menu);
  gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			    GTK_SIGNAL_FUNC(on_mount_menus),
			    (gpointer)MOUNT);
  
  menu_item = add_menu_item(_("Umount"), main_menu);
  gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			    GTK_SIGNAL_FUNC(on_mount_menus),
			    (gpointer)UMOUNT);
  
  add_menu_separator(main_menu);
  
  menu_item = add_menu_item(_("Refresh"), main_menu);
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
		     GTK_SIGNAL_FUNC(affichage_status_bar_timeout),
		     (gpointer)REFRESH);

  menu_item = add_menu_item(_("Stop"), main_menu);
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
		     GTK_SIGNAL_FUNC(affichage_status_bar_timeout),
		     (gpointer)STOP);
  
  add_menu_separator(main_menu);
  
  menu_item = add_menu_item(_("Exit"), main_menu);
  gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			    GTK_SIGNAL_FUNC(main_quit), NULL);
  
  
  /* -- Edit menu -- */

  main_item = gtk_menu_item_new_with_label(_("Edit"));
  gtk_menu_bar_append(GTK_MENU_BAR(menubar), main_item);
  gtk_widget_show(main_item);
  
  main_menu = add_menu(main_item);
  
  menu_item = add_menu_item(_("Options"), main_menu);
  gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			    GTK_SIGNAL_FUNC(options_window_create), NULL);
  
  
  /* -- Help menu -- */
  
  main_item = gtk_menu_item_new_with_label(_("Help"));
  gtk_menu_item_right_justify(GTK_MENU_ITEM(main_item));
  gtk_menu_bar_append( GTK_MENU_BAR(menubar), main_item);
  gtk_widget_show(main_item);
  
  main_menu = add_menu(main_item);
  
  menu_item = add_menu_item(_("About"), main_menu);
  gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
			    GTK_SIGNAL_FUNC(about_window), NULL);
  
  gtk_widget_show_all(menu_handle_box);
  
  return;
}


void
toolbar_create (GtkWidget *fenetre,
		GtkWidget *box)
{
  GtkWidget *toolbar;
  GtkWidget *icon;
  
  gtk_widget_realize(fenetre);
  
  tool_handle_box = gtk_handle_box_new();
  gtk_box_pack_start( GTK_BOX(box), tool_handle_box, FALSE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(tool_handle_box), "child_detached",
		     GTK_SIGNAL_FUNC(menu_signal_handle), NULL);
  
  toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
  gtk_container_add( GTK_CONTAINER(tool_handle_box), toolbar);
  /*  gtk_toolbar_set_space_size(GTK_TOOLBAR(toolbar), 20);*/
  gtk_toolbar_set_space_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_SPACE_EMPTY);
  gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_HALF);
  gtk_toolbar_set_tooltips(GTK_TOOLBAR(toolbar), TRUE);
  gtk_widget_show_all(tool_handle_box);
  
  /*--*/
  icon = create_pixmap_from_data(fenetre->window, mount_xpm);
  gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, _("Mount"),
			  NULL, icon, GTK_SIGNAL_FUNC(on_mount_menus),
			  (gpointer)MOUNT);
  /*--*/
  icon = create_pixmap_from_data(fenetre->window, umount_xpm);
  gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, _("Umount"),
			  NULL, icon, GTK_SIGNAL_FUNC(on_mount_menus),
			  (gpointer)UMOUNT);
  /*--*/  
  icon = create_pixmap_from_data(fenetre->window, refresh_xpm);
  gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, _("Refresh"),
			  NULL, icon, GTK_SIGNAL_FUNC(affichage_status_bar_timeout),
			  (gpointer)REFRESH);
  /*--*/
  icon = create_pixmap_from_data(fenetre->window, stop_xpm);
  gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, _("Stop"),
			  NULL, icon, GTK_SIGNAL_FUNC(affichage_status_bar_timeout),
			  (gpointer)STOP);
  /*--*/
  icon = create_pixmap_from_data(fenetre->window, options_xpm);
  gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, _("Options"),
			  NULL, icon, GTK_SIGNAL_FUNC(options_window_create), NULL);
  /*--*/
  icon = create_pixmap_from_data(fenetre->window, help_xpm);
  gtk_toolbar_append_item( GTK_TOOLBAR(toolbar), NULL, _("Help"),
			  NULL, icon, GTK_SIGNAL_FUNC(about_window), NULL);
  
  return;
}

void
create_popup_menu (void)
{
  GtkWidget *item;

  popup = gtk_menu_new();
  
  item = add_menu_item(_("Mount"), popup);
  gtk_signal_connect(GTK_OBJECT(item), "activate",
		     GTK_SIGNAL_FUNC(on_mount_menus),
		     (gpointer)MOUNT);
  
  item = add_menu_item(_("Umount"), popup);
  gtk_signal_connect(GTK_OBJECT(item), "activate",
		     GTK_SIGNAL_FUNC(on_mount_menus),
		     (gpointer)UMOUNT);
  
  add_menu_separator(popup);

  item = add_menu_item(_("Refresh"), popup);
  gtk_signal_connect(GTK_OBJECT(item), "activate",
		     GTK_SIGNAL_FUNC(affichage_status_bar_timeout),
		     (gpointer)REFRESH);
  
  item = add_menu_item(_("Stop"), popup);
  gtk_signal_connect(GTK_OBJECT(item), "activate",
		     GTK_SIGNAL_FUNC(affichage_status_bar_timeout),
		     (gpointer)STOP);
 
  add_menu_separator(popup);

  item = add_menu_item(_("Options"), popup);
  gtk_signal_connect(GTK_OBJECT(item), "activate",
		     GTK_SIGNAL_FUNC(options_window_create),
		     NULL);
  
  return;
}

void
on_mount_menus (GtkWidget *widget,
		gchar *action)
{
  gchar *state, *mount_point, *type;
  
  if (selected_line == -1)
    return;

  gtk_clist_get_text(GTK_CLIST(clist),
		     selected_line, STATE_COLUMN,
		     &state);
  gtk_clist_get_text(GTK_CLIST(clist),
		     selected_line, MOUNT_POINT_COLUMN,
		     &mount_point);
  gtk_clist_get_text(GTK_CLIST(clist),
		     selected_line, FILESYSTEM_TYPE_COLUMN,
		     &type);

  if (atoi(action) == atoi(state))
    return;
  
  if (!strcmp(action, MOUNT))
    mount_fs(NULL, mount_point, type);
  else
    umount_fs(NULL, mount_point, type);
  
  return;
}
