/*Written with Emacs.*/
/*
  GtkDiskFree shows free space on your mounted partitions.
  Copyright (C) 2001-2002  Dj-Death (Landwerlin Lionel)
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "main.h"
#include "callbacks.h"
#include "interface.h"
#include "configure.h"
#include "about.h"
#include "diskfree.h"
#include "options.h"
#include "widgets.h"

void
load_preferences_options (preferences_t *from,
			  preferences_t *to)
{
  gint i;
  
  to->blocks = from->blocks;
  to->virtual_blocks = from->virtual_blocks;
  to->not_mounted_blocks = from->not_mounted_blocks;
  to->menubar = from->menubar;
  to->toolbar = from->toolbar;
  to->time = from->time;
  to->show_all_columns = from->show_all_columns;
  to->sort_by = from->sort_by;
  to->sort_type = from->sort_type;
  to->color_percent = from->color_percent;
  to->degrade = from->degrade;
  for (i=0 ; i<2 ; i++)
    {
      to->window_position[i] = from->window_position[i];
      to->window_size[i] = from->window_size[i];
      to->color_levels[i] = from->color_levels[i];
    }
  for (i=0 ; i<3 ; i++)
    {
      to->color1[i] = from->color1[i];
      to->color2[i] = from->color2[i];
    }
  for (i=0 ; i<NBRE_COLONNES ; i++)
    to->hided_columns[i] = from->hided_columns[i];
  
  return;
}

gboolean
preferences_free (void)
{
  g_free(preferences);
  
  return TRUE; 
}

gboolean
ok_button_clicked (GtkWidget *widget,
		   GtkWidget **window)
{
  color_update = 1;
  preferences->time = atof(gtk_entry_get_text(GTK_ENTRY(entry_options[0])));

  load_preferences_options(preferences, options);
  window_destroy(*window, window);
  preferences_free();
  
  show_hide_menus();
  show_hide_columns();

  clist_main_update();
  affichage_status_bar_block_size();
  
  return TRUE;
}

gboolean
apply_button_clicked (void)
{
  color_update = 1;
  preferences->time = atof(gtk_entry_get_text(GTK_ENTRY(entry_options[0])));
  
  load_preferences_options(preferences, options);
  
  show_hide_menus();
  show_hide_columns();
  
  clist_main_update();
  affichage_status_bar_block_size();
  
  return TRUE;
}

gboolean
save_button_clicked (void)
{
  preferences->time = atof(gtk_entry_get_text(GTK_ENTRY(entry_options[0])));
  write_config_file(preferences);
  
  return TRUE;
}

gboolean
color_window_update (GtkWidget *button,
		     GtkWidget *sel)
{
  gint i;
  gdouble *color;
  gdouble c[3];
  GtkWidget *pixmap;
  GdkWindow *window;
  
  color = (gdouble *) gtk_object_get_data(GTK_OBJECT(button),
					  "color");
  pixmap = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(button),
					     "pixmap");
  window = gtk_widget_get_parent_window(pixmap);
  gtk_color_selection_get_color(GTK_COLOR_SELECTION(sel), c);
  
  for (i=0 ; i<3 ; i++)
    color[i] = c[i];
  set_pixmap_color(window, pixmap, PREVIEW_W, PREVIEW_H, color);
  
  return TRUE;
}

gboolean
color_window1_create (GtkWidget *button, GtkWidget *pixbox)
{
  static GtkWidget *window = NULL;
  
  if (window != NULL)
    {
      window_destroy(window, &window);
      return FALSE;
    }
  
  color_window_create(_("Choose the first color"),
		      preferences->color1,
		      &window, pixbox);
  
  return TRUE;
}

gboolean
color_window2_create (GtkWidget *button, GtkWidget *pixbox)
{
  static GtkWidget *window = NULL;
  
  if (window != NULL)
    {
      window_destroy(window, &window);
      return FALSE;
    }
  
  color_window_create(_("Choose the second color"),
		      preferences->color2,
		      &window, pixbox);
  
  return TRUE;
}

void
color_window_create (gchar *title, gdouble *color,
		     GtkWidget **window, GtkWidget *pixbox)
{
  gint i;
  gdouble c[3];
  GtkColorSelection *sel;
  GtkWidget *button;
  
  for (i=0 ; i<3 ; i++)
    c[i] = color[i];
  
  *window = gtk_dialog_new();
  gtk_window_set_policy(GTK_WINDOW(*window), 0, 0, 0);
  gtk_window_set_title(GTK_WINDOW(*window), title);
  gtk_signal_connect(GTK_OBJECT(*window),
		     "destroy",
		     GTK_SIGNAL_FUNC(window_destroy),
		     window);
  
  sel = (GtkColorSelection *) gtk_color_selection_new();
  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(*window)->vbox),
			      GTK_WIDGET(sel));
  gtk_color_selection_set_color(sel, color);
  gtk_widget_show(GTK_WIDGET(sel));
    
  button = gtk_button_new_with_label(_("OK"));
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_object_set_data(GTK_OBJECT(button), "color", color);
  gtk_object_set_data(GTK_OBJECT(button), "pixmap", pixbox);
  gtk_signal_connect(GTK_OBJECT(button),
		     "clicked",
		     GTK_SIGNAL_FUNC(color_window_update),
		     GTK_OBJECT(sel));
  gtk_signal_connect_after(GTK_OBJECT(button),
			   "clicked",
			   GTK_SIGNAL_FUNC(window_destroy),
			   window);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(*window)->action_area),
		     button, TRUE, TRUE, 0);
  gtk_widget_show(button);
  
  button = gtk_button_new_with_label(_("Cancel"));
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_signal_connect(GTK_OBJECT(button),
		     "clicked",
		     GTK_SIGNAL_FUNC(window_destroy),
		     window);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(*window)->action_area),
		     button, TRUE, TRUE, 0); 
  gtk_widget_show(button);
  gtk_widget_grab_default(button);
  
  gtk_widget_show(*window);
  
  return;
}

gboolean
options_window_create (void)
{
  gint index;
  gchar *entry;

  static GtkWidget *window = NULL;
  GtkWidget *h_box;
  GtkWidget *button;

  GtkWidget *ctree;
  GtkCTreeNode *top_insert;
  GtkCTreeNode *child_insert;

  GtkWidget *frame;
  GtkWidget *notebook;
  GtkWidget *p_box;
  
  GtkWidget *separateur;
  GtkWidget *hbox1, *vbox1, *hbox_columns, *vbox_color_levels, *vbox_degrade_colors;
  
  GtkWidget *time_entry;
  GtkWidget *label;
  GtkWidget *check_button, *radio_button;
  
  GtkWidget *src;
  GtkWidget *pixmap;
  
  if (window != NULL)
    {
      window_destroy(window, &window);
      return FALSE;
    }
  preferences = g_malloc(sizeof(preferences_t));
  load_preferences_options(options, preferences);
  
  /*-- Window --*/
  
  window = gtk_dialog_new();
  gtk_window_set_policy(GTK_WINDOW(window), 0, 0, 0);
  gtk_window_set_title(GTK_WINDOW(window),
		       _("Preferences"));
  gtk_widget_realize(GTK_WIDGET(window));
  gtk_container_set_border_width(GTK_CONTAINER(window), 5);
  gtk_signal_connect(GTK_OBJECT(window),
		     "destroy",
		     GTK_SIGNAL_FUNC(window_destroy),
		     &window);


  /*-- Tree --*/
  
  h_box = gtk_hbox_new(FALSE, 6);
  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(window)->vbox), h_box);
  gtk_widget_show(h_box);

  ctree = gtk_ctree_new(1, 0);
  gtk_ctree_set_indent(GTK_CTREE(ctree), 15);
  gtk_widget_set_usize(ctree,140,0);
  gtk_box_pack_start(GTK_BOX(h_box), ctree, FALSE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(ctree),
		     "tree_select_row",
		     GTK_SIGNAL_FUNC(ctree_clicked),
		     NULL);  
  gtk_widget_show(ctree);

  /*-- Notebook --*/
  
  frame=gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(frame),
			    GTK_SHADOW_IN);
  gtk_box_pack_start(GTK_BOX(h_box),
		     frame, TRUE, TRUE, 0);
  gtk_widget_show(frame);

  notebook = gtk_notebook_new();
  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook),
			     FALSE);
  gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook),
			       FALSE);
  gtk_container_add(GTK_CONTAINER(frame),
		    notebook);
  gtk_object_set_user_data(GTK_OBJECT(ctree),
			   notebook);
  gtk_widget_show(notebook);
  /*-- Tree titles --*/
  
  index = 0;

  /*-- Blocks --*/
  p_box = add_node(GTK_NOTEBOOK(notebook),
		   _("Blocks"),
		   GTK_CTREE(ctree),
		   _("Blocks"),
		   NULL,
		   &top_insert,
		   index);
  
  gtk_ctree_select(GTK_CTREE(ctree), top_insert);
  index++;

  add_check(_("Include filesystems contain 0 blocks"),
	    &(preferences->virtual_blocks),
	    p_box);
  add_check(_("Include not mounted filesystems"),
	    &(preferences->not_mounted_blocks),
	    p_box);

  separateur = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(p_box), separateur, FALSE, FALSE, 0);
  gtk_widget_show(separateur);
  
  radio_button = add_radio (_("Human readable"), &preferences->blocks,
			    "104", NULL, p_box);
  radio_button = add_radio(_("Gigabytes"), &preferences->blocks,
			   "103", radio_button, p_box);
  radio_button = add_radio(_("Megabytes"), &preferences->blocks,
			   "109", radio_button, p_box);
  radio_button = add_radio(_("Kilobytes"), &preferences->blocks,
			   "107", radio_button, p_box);

  separateur = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(p_box), separateur, FALSE, FALSE, 0);
  gtk_widget_show(separateur);

  hbox1 = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(p_box), hbox1, FALSE, FALSE, 0);
  gtk_widget_show(hbox1);
  
  label = gtk_label_new(_("refreshments :"));
  gtk_box_pack_start_defaults(GTK_BOX(hbox1), label);
  gtk_widget_show(label);
  
  entry_options = g_malloc(1 * sizeof(GtkWidget *));  
  time_entry = gtk_entry_new();
  entry_options[0] = time_entry;
  gtk_box_pack_start(GTK_BOX(hbox1), time_entry, FALSE, FALSE, 0);
  gtk_widget_show(time_entry);
  
  entry = g_strdup_printf("%f", preferences->time); 
  gtk_entry_set_text(GTK_ENTRY(time_entry), entry);
  g_free(entry);
    
  gtk_widget_show(p_box);
  
  
  /*-- Columns --*/
  p_box = add_node(GTK_NOTEBOOK(notebook),
		   _("Columns"),
		   GTK_CTREE(ctree),
		   _("Columns"),
		   NULL,
		   &top_insert,
		   index);
  index++;
  
  
  hbox_columns = gtk_hbox_new(FALSE, 0);

  check_button = add_check(_("Show all columns"),
			   &(preferences->show_all_columns),
			   p_box);
  gtk_signal_connect_object(GTK_OBJECT(check_button),
			    "toggled",
			    GTK_SIGNAL_FUNC(set_widget_sensitivity),
			    GTK_OBJECT(hbox_columns));
  
  separateur = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(p_box), separateur, FALSE, FALSE, 0);
  gtk_widget_show(separateur);

  gtk_box_pack_start(GTK_BOX(p_box), hbox_columns, FALSE, FALSE, 0);
  gtk_widget_show(hbox_columns);
  
  if (preferences->show_all_columns)
    gtk_widget_set_sensitive(hbox_columns, FALSE);

  vbox1 = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_columns), vbox1, FALSE, FALSE, 0);
  gtk_widget_show(vbox1);

  add_check(_("Filesystem"),
	    &(preferences->hided_columns[FILESYSTEM_COLUMN]),
	    vbox1);
  add_check(_("Size"),
	    &(preferences->hided_columns[SIZE_COLUMN]),
	    vbox1);
  add_check(_("Used space"),
	    &(preferences->hided_columns[USED_SPACE_COLUMN]),
	    vbox1);
  add_check(_("Free space"),
	    &(preferences->hided_columns[FREE_SPACE_COLUMN]),
	    vbox1);
  add_check(_("Used percent"),
	    &(preferences->hided_columns[USED_PERCENT_COLUMN]),
	    vbox1);

  separateur = gtk_vseparator_new();
  gtk_box_pack_start(GTK_BOX(hbox_columns), separateur, FALSE, FALSE, 0);
  gtk_widget_show(separateur);

  vbox1 = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_columns), vbox1, FALSE, FALSE, 0);
  gtk_widget_show(vbox1);
  
  add_check(_("Free percent"),
	    &(preferences->hided_columns[FREE_PERCENT_COLUMN]),
	    vbox1);
  add_check(_("Filesystem type"),
	    &(preferences->hided_columns[FILESYSTEM_TYPE_COLUMN]),
	    vbox1);
  add_check(_("Mount option's"),
	    &(preferences->hided_columns[MOUNT_OPTIONS_COLUMN]),
	    vbox1);
  add_check(_("Mount point"),
	    &(preferences->hided_columns[MOUNT_POINT_COLUMN]),
	    vbox1);
  add_check(_("Capacity"),
	    &(preferences->hided_columns[CAPACITY_COLUMN]),
	    vbox1);
  
  gtk_widget_show(p_box);
  
  /*-- Apparence --*/
  p_box = add_node(GTK_NOTEBOOK(notebook),
		   _("Style"),
		   GTK_CTREE(ctree),
		   _("Style"),
		   NULL,
		   &top_insert,
		   index);
  index++;
  
  add_check(_("Show menubar"),
	    &(preferences->menubar),
	    p_box);
  add_check(_("Show toolbar"),
	    &(preferences->toolbar),
	    p_box);
  
  gtk_widget_show(p_box);

  /*-- Jauge --*/
  p_box = add_node(GTK_NOTEBOOK(notebook),
		   _("Capacity"),
		   GTK_CTREE(ctree),
		   _("Capacity"),
		   NULL,
		   &top_insert,
		   index);
  index++;
  
  vbox_color_levels = gtk_vbox_new(FALSE, 0);
  vbox_degrade_colors = gtk_vbox_new(FALSE, 0);
  
  add_check(_("Show percent in capacity column"),
	    &(preferences->color_percent),
	    p_box);
  check_button = add_check(_("Make degrades"),
			   &(preferences->degrade),
			   p_box);
  gtk_signal_connect_object(GTK_OBJECT(check_button),
			    "toggled",
			    GTK_SIGNAL_FUNC(set_widget_sensitivity),
			    GTK_OBJECT(vbox_color_levels));
  gtk_signal_connect_object(GTK_OBJECT(check_button),
			    "toggled",
			    GTK_SIGNAL_FUNC(set_widget_sensitivity),
			    GTK_OBJECT(vbox_degrade_colors));
  gtk_widget_show(check_button);
  
  add_vhseparator(p_box, HORIZONTAL);
  
  gtk_box_pack_start(GTK_BOX(p_box), vbox_color_levels, FALSE, FALSE, 0);
  gtk_widget_show(vbox_color_levels);
  
  gtk_box_pack_start(GTK_BOX(p_box), vbox_degrade_colors, FALSE, FALSE, 0);
  gtk_widget_show(vbox_degrade_colors);

  if (preferences->degrade)
    set_widget_sensitivity(vbox_color_levels);
  else
    set_widget_sensitivity(vbox_degrade_colors);
  
  add_scale(_("Level for the green color"), &(preferences->color_levels[0]),
	    vbox_color_levels);
  add_scale(_("Level for the red color"), &(preferences->color_levels[1]),
	    vbox_color_levels);
  
  add_vhseparator(p_box, HORIZONTAL);
  /* First Color */
  button = add_button(NULL, vbox_degrade_colors, FALSE);
  
  hbox1 = gtk_hbox_new(TRUE, 0);
  gtk_container_add(GTK_CONTAINER(button), hbox1);
  gtk_widget_show(hbox1);
  
  label = gtk_label_new(_("First Color :"));
  gtk_box_pack_start(GTK_BOX(hbox1), label,
		     FALSE, FALSE, 0);
  gtk_widget_show(label);
  
  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
  gtk_box_pack_start(GTK_BOX(hbox1), frame, FALSE, FALSE, 0);
  gtk_widget_show(frame);
  
  pixmap = create_pixmap_from_color(window->window,
				    PREVIEW_W, PREVIEW_H,
				    preferences->color1);
  gtk_container_add(GTK_CONTAINER(frame), pixmap);
  gtk_widget_show(pixmap);
  gtk_signal_connect(GTK_OBJECT(button),
		     "clicked",
		     GTK_SIGNAL_FUNC(color_window1_create),
		     GTK_OBJECT(pixmap));
  /* Second Color */
  button = add_button(NULL, vbox_degrade_colors, FALSE);
  
  hbox1 = gtk_hbox_new(TRUE, 0);
  gtk_container_add(GTK_CONTAINER(button), hbox1);
  gtk_widget_show(hbox1);
  
  label = gtk_label_new(_("Second Color :"));
  gtk_box_pack_start(GTK_BOX(hbox1), label,
		     FALSE, FALSE, 0);
  gtk_widget_show(label);
  
  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
  gtk_box_pack_start(GTK_BOX(hbox1), frame, FALSE, FALSE, 0);
  gtk_widget_show(frame);
  
  pixmap = create_pixmap_from_color(window->window,
				    PREVIEW_W, PREVIEW_H,
				    preferences->color2);
  gtk_container_add(GTK_CONTAINER(frame), pixmap);
  gtk_widget_show(pixmap);
  gtk_signal_connect(GTK_OBJECT(button),
		     "clicked",
		     GTK_SIGNAL_FUNC(color_window2_create),
		     GTK_OBJECT(pixmap));
  
  gtk_widget_show(p_box);
  /*-- Info --*/
  
  p_box = add_node(GTK_NOTEBOOK(notebook),
		   _("Informations"),
		   GTK_CTREE(ctree),
		   _("Informations"),
		   NULL,
		   &top_insert,
		   index);
  index++;
  
  about_gtkdiskfree(window, p_box);
  gtk_widget_show(p_box);

  /*-- Legal --*/
  
  p_box = add_node(GTK_NOTEBOOK(notebook),
		   _("Legal"),
		   GTK_CTREE(ctree),
		   _("Legal"),
		   top_insert,
		   &child_insert,
		   index);
  index++;
  
  src = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(src),
				 GTK_POLICY_NEVER,
				 GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start_defaults(GTK_BOX(p_box), src);
  gtk_widget_show(src);

  legal_gtkdiskfree(src);

  gtk_widget_show(p_box);

  /*----Buttons----*/

  button = gtk_button_new_with_label(_("OK"));
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_signal_connect(GTK_OBJECT(button),
		     "clicked",
		     GTK_SIGNAL_FUNC(ok_button_clicked),
		     &window);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area),
		     button, TRUE, TRUE, 0);
  gtk_widget_show(button);

  button = gtk_button_new_with_label(_("Apply"));
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_signal_connect(GTK_OBJECT(button),
		     "clicked",
		     GTK_SIGNAL_FUNC(apply_button_clicked),
		     NULL);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area),
		     button, TRUE, TRUE, 0); 	
  gtk_widget_show(button);
  
  button = gtk_button_new_with_label(_("Save"));
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_signal_connect(GTK_OBJECT(button),
		     "clicked",
		     GTK_SIGNAL_FUNC(save_button_clicked),
		     NULL);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area),
		     button, TRUE, TRUE, 0); 	
  gtk_widget_show(button);
  
  button = gtk_button_new_with_label(_("Cancel"));
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_signal_connect(GTK_OBJECT(button),
		     "clicked",
		     GTK_SIGNAL_FUNC(window_destroy),
		     &window);
  gtk_signal_connect(GTK_OBJECT(button),
		     "clicked",
		     GTK_SIGNAL_FUNC(preferences_free),
		     NULL);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area),
		     button, TRUE, TRUE, 0); 	
  gtk_widget_show(button);
  gtk_widget_grab_default(button);

  gtk_widget_show(window);
  
  return TRUE;
}
