/*Written with Emacs.*/
/*
  GtkDiskFree shows free space on your mounted partitions.
  Copyright (C) 2001-2002  Dj-Death (Landwerlin Lionel)
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "main.h"
#include "options.h"
#include "widgets.h"

/* make part */

void set_pixmap_color (GdkWindow *window,
		       GtkWidget *pixmap,
		       gint width, gint height,
		       gdouble *color)
{  
  GdkPixmap *pix;
  GdkGC *gc;
  GdkColormap *palette;
  GdkColor c;
  
  pix = gdk_pixmap_new(window, width, height, -1);
  
  c.red = (gint) (color[0] * 65535);
  c.green = (gint) (color[1] * 65535);
  c.blue = (gint) (color[2] * 65535);
  
  gc = gdk_gc_new(window);
  palette = gdk_window_get_colormap(window);
  gdk_color_alloc(palette, &c);
  gdk_gc_set_foreground(gc, &c);
  
  gdk_draw_rectangle(pix, gc, TRUE, 0, 0,
		     width, height);
  gtk_pixmap_set(GTK_PIXMAP(pixmap), pix, NULL);
  gdk_pixmap_unref(pix);
  
  return;
}

GtkWidget *
create_pixmap_from_color (GdkWindow *window,
			  gint width, gint height,
			  gdouble *color)
{
  GdkPixmap *pix;
  GdkGC *gc;
  GdkColor c;
  GdkColormap *palette;
  GtkWidget *pixmap;
  
  c.red = (gint) (color[0] * 65535);
  c.green = (gint) (color[1] * 65535);
  c.blue = (gint) (color[2] * 65535);
  
  gc = gdk_gc_new(window);
  palette = gdk_window_get_colormap(window);
  gdk_color_alloc(palette, &c);
  gdk_gc_set_foreground(gc, &c);
  
  pix = gdk_pixmap_new(window, width, height, -1);
  gdk_draw_rectangle(pix, gc, TRUE, 0, 0,
		     width, height);
  pixmap = gtk_pixmap_new(pix, NULL);
  gdk_pixmap_unref(pix);
  
  return pixmap;
}  

GtkWidget *
create_pixmap_from_data (GdkWindow *window,
			 gchar **icon)
{
  GtkWidget *pix;
  GdkPixmap *pixmap;
  GdkBitmap *masque;
  GdkColor trans;
  
  pixmap = gdk_pixmap_create_from_xpm_d(window, &masque,
					&trans, icon);
  pix = gtk_pixmap_new(pixmap, masque);
  gdk_pixmap_unref(pixmap);
  gdk_pixmap_unref(masque);
  
  return pix;
}

void
add_vhseparator (GtkWidget *box, gint orientation)
{
  GtkWidget *sep;
  
  if (orientation == HORIZONTAL)
    sep = gtk_hseparator_new();
  else
    sep = gtk_vseparator_new();
  gtk_box_pack_start_defaults(GTK_BOX(box), sep);
  gtk_widget_show(sep);
  
  return;
}

void
add_menu_separator (GtkWidget *menu)
{
  GtkWidget *item;
  
  item = gtk_menu_item_new();
  gtk_menu_append(GTK_MENU(menu), item);
  gtk_widget_set_sensitive(item, FALSE);
  gtk_widget_show(item);
  
  return;
}

GtkWidget *
add_button (gchar *title, GtkWidget *box,
	    gint expand)
{
  GtkWidget *button;
  
  if (title != NULL)
    button = gtk_button_new_with_label(title);
  else
    button = gtk_button_new();
  if (expand)
    gtk_box_pack_start_defaults(GTK_BOX(box),
				button);
  else
    gtk_box_pack_start(GTK_BOX(box), button,
		       FALSE, FALSE, 0);
  gtk_widget_show(button);
  
  return button;
}

GtkWidget *
add_menu (GtkWidget *item)
{
  GtkWidget *menu;
  
  menu = gtk_menu_new();
  if (item != NULL)
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
  gtk_widget_show(menu);
  
  return menu;
}

GtkWidget *
add_menu_item (gchar *title,
	       GtkWidget *menu)
{
  GtkWidget *item;
  
  item = gtk_menu_item_new_with_label(title);
  gtk_menu_append(GTK_MENU(menu), item);
  gtk_widget_show(item);
  
  return item;
}

GtkWidget *
add_scale (gchar *title, gint *value,
	   GtkWidget *box)
{
  GtkObject *adj;
  GtkWidget *label;
  GtkWidget *scale;
  
  label = gtk_label_new(title);
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
  gtk_widget_show(label);
  
  adj = gtk_adjustment_new((gfloat) *value,
			   0.0, 100.0,
			   1.0, 5.0, 1.0);
  gtk_signal_connect(adj, "value_changed",
		     GTK_SIGNAL_FUNC(scale_changed), value);
  scale = gtk_hscale_new(GTK_ADJUSTMENT(adj));
  gtk_scale_set_draw_value(GTK_SCALE(scale), TRUE);
  gtk_scale_set_value_pos(GTK_SCALE(scale), GTK_POS_BOTTOM);
  gtk_box_pack_start(GTK_BOX(box), scale, FALSE, FALSE, 0);
  gtk_widget_show(scale);
  
  return scale;
}

GtkWidget *
add_check (gchar *title, gint *value,
	   GtkWidget *box)
{
  GtkWidget *check;
  
  check = gtk_check_button_new_with_label(title);
  if (*value)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), TRUE);
  gtk_box_pack_start(GTK_BOX(box), check, FALSE, FALSE, 0);
  gtk_signal_connect(GTK_OBJECT(check), "toggled",
		     GTK_SIGNAL_FUNC(check_button_clicked),
		     (gpointer) value);
  gtk_widget_show(check);
  
  return check;
}

GtkWidget *
add_radio (gchar *title, gint *value,
	   gchar *signal, GtkWidget *from,
	   GtkWidget *box)
{
  GtkWidget *radio;
  
  if (from != NULL)
    radio = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(from),
							title);
  else
    radio = gtk_radio_button_new_with_label_from_widget(NULL, title);
  gtk_box_pack_start(GTK_BOX(box), radio, FALSE, FALSE ,0);
  if (*value == atoi(signal))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), TRUE);
  gtk_object_set_data(GTK_OBJECT(radio), "value", value);
  gtk_signal_connect(GTK_OBJECT(radio), "toggled",
		     GTK_SIGNAL_FUNC(radio_button_clicked),
		     (gpointer) signal);
  gtk_widget_show(radio);
  
  return radio;
}

/*-- Bas sur la fenetre de prfrences de 'The GIMP' --*/
GtkWidget *
add_node (GtkNotebook *notebook,
	  gchar *notebook_label,
	  GtkCTree *ctree,
	  gchar *ctree_label,
	  GtkCTreeNode *parent,
	  GtkCTreeNode **new_node,
	  gint index)
{
  gchar *text_label[1];
  GtkWidget *vbox;
  GtkWidget *frame;
  GtkWidget *label;
  
  vbox = gtk_vbox_new(FALSE, 0);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, NULL);
  
  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
  gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 0);
  gtk_widget_show(frame);

  label = gtk_label_new(notebook_label);
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_misc_set_padding(GTK_MISC(label), 2, 1);
  gtk_container_add(GTK_CONTAINER(frame), label);
  gtk_widget_show(label);

  text_label[0] = ctree_label;
  *new_node = gtk_ctree_insert_node (ctree,
				     parent,
				     NULL,
				     text_label,
				     0,
				     NULL,
				     NULL,
				     NULL,
				     NULL,
				     FALSE,
				     FALSE);
  gtk_ctree_node_set_row_data(ctree,
			      *new_node,
			      (gpointer) index);
  return vbox;
}

gboolean
window_destroy(GtkWidget *widget,
	       GtkWidget **window)
{
  static gint ret = FALSE;
  
  if (ret)
    return FALSE;
  
  ret = TRUE;
  gtk_widget_destroy(*window);
  gtk_widget_destroyed(*window, window);
  ret = FALSE;
  
  return TRUE;
  
}

/* signal part */

gboolean
scale_changed (GtkAdjustment *adj,
	       gint *value)
{
  *value = (gint) adj->value;
  
  return TRUE;
}

gboolean
radio_button_clicked (GtkWidget *button,
		      gpointer message)
{
  gint *value;
  
  value = (gint *) gtk_object_get_data(GTK_OBJECT(button), "value");
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
    *value = atoi((gchar *) message);
  
  return TRUE;
}

gboolean
check_button_clicked (GtkWidget *button,
		      gint *value)
{
  value = (gint *) value;
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
    *value = 1;
  else
    *value = 0;
  
  return TRUE;
}

gboolean
ctree_clicked (GtkWidget *ctree,
	       GtkCTreeNode *node)
{
  GtkNotebook *notebook;
  gint page;
  
  notebook = (GtkNotebook*) gtk_object_get_user_data(GTK_OBJECT(ctree));
  page = (gint) gtk_ctree_node_get_row_data(GTK_CTREE(ctree), node);
  gtk_notebook_set_page(notebook, page);
  
  return TRUE;
}

void
set_widget_sensitivity (GtkWidget *widget)
{
  if (GTK_WIDGET_SENSITIVE(widget))
    gtk_widget_set_sensitive(widget, FALSE);
  else
    gtk_widget_set_sensitive(widget, TRUE);
  
  return;
}
